from sqlalchemy import Column, Integer, String, DateTime
# from sqlalchemy.orm import relationship

from database import Base

import secrets
from Crypto.Util.Padding import pad, unpad
from Crypto.Cipher import AES
from base64 import b64encode, b64decode
from datetime import datetime, timedelta
import hashlib


class Secret(Base):
    __tablename__ = "secrets"

    # id = Column(Integer, primary_key=True, index=True)
    uid = Column(String, primary_key=True, index=True)
    secret = Column(String)
    iv = Column(String)
    reads = Column(Integer, default=1)
    expire = Column(DateTime, default=datetime.now()+timedelta(days=1))

    @classmethod
    def genPassword(cls, len=32) -> str:
        """
        Return a urlsafe password of specified length
        """
        return secrets.token_urlsafe(len)

    @classmethod
    def encrypt(cls, text, password):
        """
        Encrypt `text` using `password` as salt for a sha256 hash
        Return:
            a Secret object
        """

        key = hashlib.sha256(password.encode('utf8')).digest()
        cipher = AES.new(key, AES.MODE_CBC)
        secret = cipher.encrypt(pad(text.encode('utf8'), AES.block_size))

        return Secret(b64encode(secret), b64encode(cipher.iv))

    def __init__(self, secret, iv, reads=1) -> None:
        """
        Store the secret and the IV, needed for decryption
        Secret and IV are encoded in base64 mode
        """
        self.secret = secret
        self.iv = iv
        self.uid = secrets.token_urlsafe(16)
        self.reads = reads

    def __str__(self) -> str:

        return self.secret.decode('utf8')

    def decrypt(self, password) -> bytes:
        """
        Decrypt the secret, back to clear text.
        """

        key = hashlib.sha256(password.encode('utf8')).digest()

        iv = b64decode(self.iv)  # type: ignore
        cipher = AES.new(key, AES.MODE_CBC, iv=iv)
        secret = b64decode(self.secret)  # type: ignore
        clear = cipher.decrypt(secret)
        return unpad(clear, AES.block_size)
