from fastapi import APIRouter, Depends, HTTPException
from models import secrets, schemas
from database import get_db
from sqlalchemy.orm import Session

router = APIRouter()


@router.post("/secrets/", response_model=schemas.Secret)
def create_secret(secret: schemas.SecretCreate, db: Session = Depends(get_db)):
    db_secret = secrets.Secret(secret=secret.secret,
                               iv=secret.iv,
                               reads=secret.reads)  # type: ignore
    db.add(db_secret)
    db.commit()
    db.refresh(db_secret)
    return db_secret


# @router.post("/secrets/encrypt/{text:str}/{password:str}",
@router.post("/secrets/encrypt", response_model=schemas.SecretClear)
def encrypt(secret: schemas.SecretClearCreate, db: Session = Depends(get_db)):
    db_secret = secrets.Secret.encrypt(secret.text, secret.password)
    db_secret.reads = secret.reads

    db.add(db_secret)
    db.commit()
    db.refresh(db_secret)
    ret_secret = db_secret.__dict__
    ret_secret['password'] = secret.password
    return ret_secret


# @router.get("/secrets/", response_model=list[schemas.Secret])
# def read_secrets(skip: int = 0, limit: int = 100,
#                  db: Session = Depends(get_db)):
#     return db.query(secrets.Secret).offset(skip).limit(limit).all()


@router.get("/secrets/{uid:str}", response_model=schemas.Secret)
def read_secret(uid, db: Session = Depends(get_db)):
    secret = db.query(secrets.Secret).filter(
        secrets.Secret.uid == uid).first()

    if secret:
        secret.reads -= 1  # type: ignore
        if secret.reads == 0:  # type: ignore
            db.delete(secret)
        db.commit()
        return secret
    raise HTTPException(status_code=404, detail="Secret not found")


@router.post("/secrets/decrypt/{uid:str}/{password:str}")
def decrypt_secret(uid, password, db: Session = Depends(get_db)):
    secret = db.query(secrets.Secret).filter(
        secrets.Secret.uid == uid).first()

    if secret:
        clear = secret.decrypt(password)
        secret.reads -= 1  # type: ignore
        if secret.reads == 0:  # type: ignore
            db.delete(secret)
        db.commit()
        return {'text': clear}
    raise HTTPException(status_code=404, detail="Secret not found")


@router.delete("/secrets/{uid:str}", response_model=schemas.Secret)
def delete_secret(uid, db: Session = Depends(get_db)):
    secret = db.query(secrets.Secret).filter(
        secrets.Secret.uid == uid).first()

    if secret:
        db.delete(secret)
        db.commit()
        raise HTTPException(status_code=200,
                            detail="Secret deleted succesfully")
    raise HTTPException(status_code=404, detail="Secret not found")
